<?php
if (!defined('ABSPATH')) { exit; }

class ATEC_wpmc_activation { function __construct() {
	
	if (!defined('ATEC_TOOLS_INC')) require_once(__DIR__.'/atec-tools.php');
	
	$options					= atec_create_options('atec_WPMC_settings',['cache','debug','salt','type','minify','log','idx','charset','custom'],['minify']);
	$options['salt']			= hash('crc32', get_bloginfo(), FALSE);
	$options['charset'] 	= get_bloginfo('charset');
	$options['cache']		= false;
	$extensions				= atec_create_options('atec_WPMC_extensions',['apcu','redis','memcached','sqlite','database','file']);

	global $wpdb;
	$table=$wpdb->base_prefix.'mega_cache';
	$options['idx']=$table.'_idx';

	$notice = [];
	$success = true;
	// @codingStandardsIgnoreStart
	$wpdb->suppress_errors(true);
	$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $table ) );
	$engine=' ENGINE = InnoDB CHARSET=utf8mb4 COLLATE utf8mb4_general_ci';
	if (!$wpdb->get_var($query)==$table) 
	{
		$query=' (`id` INT NOT NULL AUTO_INCREMENT, `key` TEXT NOT NULL, `content` MEDIUMBLOB, PRIMARY KEY (`id`))'.$engine;
		$success = $success && $wpdb->query("CREATE TABLE {$table} ".$query)!==false;
	}
	if (!$wpdb->get_var($wpdb->prepare('SHOW TABLES LIKE %s', $options['idx']))==$table) 
	{
		$query=' (`id` INT NOT NULL AUTO_INCREMENT, `key` TEXT, `suffix` VARCHAR(1), `pid` MEDIUMINT, `size` MEDIUMINT, PRIMARY KEY (`id`))'.$engine.' ROW_FORMAT=COMPRESSED';
		$success = $success && $wpdb->query("CREATE TABLE {$options['idx']} ".$query)!==false;
	}
	$wpdb->suppress_errors(false);
	// @codingStandardsIgnoreEnd
	if (!$success) atec_notice($notice, 'error', 'Failed to create plugin tables.');
	$extensions['database'] = $table;

	$uploadDir = atec_get_upload_dir('mega-cache');
	$success = atec_mkdir_if_not_exists($uploadDir);
	if ($success)
	{
		global $wp_filesystem; 	WP_Filesystem();

		$arr=['index.php'=>'index.php', 'htaccess.txt'=>'.htaccess'];
		atec_copy_install_files(__DIR__,$uploadDir,$arr,$success);
		$dirArr=['cache','cache/file','cache/sqlite'];
		foreach($dirArr as $dir) 
		{
			$subDir = $uploadDir.DIRECTORY_SEPARATOR.$dir;
			$wp_filesystem->mkdir($subDir);
			atec_copy_install_files(__DIR__,$subDir,$arr,$success);
		}
		$extensions['file'] = $uploadDir.'/cache/file/';		
	}
	
	if (!$success)
	{
		atec_notice($notice, 'error', 'Failed to create uploads folder and files.');
		$uploadDir = '';
		update_option( 'atec_wpb_debug', $notice, false);
	}

	if ($uploadDir!=='' && class_exists('SQLite3'))
	{
		$sqlitePath=$uploadDir.'/cache/sqlite/mega_cache.sqlite';
		$sqlite = @new SQLite3($sqlitePath, SQLITE3_OPEN_READWRITE | SQLITE3_OPEN_CREATE);
		if (!$sqlite) atec_notice($notice, 'warning', 'Failed to create SQLite database.');
		else 
		{ 
			$sqlite->exec('CREATE TABLE IF NOT EXISTS "atec_wpmc" ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, "key" CHARACTER(32) NOT NULL, "content" BLOB)'); 
			$sqlite->close();
			$extensions['sqlite']=$sqlitePath;
		}
	}
	
	if (extension_loaded('mongodb')) { $mdbSettings = array('host'=>'localhost', 'port'=>27017); }
	else $mdbSettings = array();
	$extensions['mongodb'] = $mdbSettings;
	
	if (class_exists('redis')) { $rdSettings = array('host'=>'localhost', 'port'=>6379, 'pwd'=>'', 'unix'=>''); }
	else $rdSettings = array();
	$extensions['redis'] = $rdSettings;

	if (class_exists('Memcached')) { $mcSettings = array('host'=>'localhost', 'port'=>11211, 'unix'=>''); }
	else $mcSettings = array();
	$extensions['memcached'] = $mcSettings;
	
	$slug='atec_WPMC';
	update_option($slug.'_settings',$options);
	update_option($slug.'_extensions',$extensions,false);
	update_option($slug.'_active',['file'=>true],false);
	
	if ($uploadDir!=='')
	{
		$arr=array($slug.'_settings'=>$options, $slug.'_extensions'=>$extensions);
		$wp_filesystem->put_contents($uploadDir.'/mega-cache.json',wp_json_encode($arr));
	}
		
	if (!empty($notice)) update_option('atec_wpmc_debug', $notice, false);
	
}} 
new ATEC_wpmc_activation();
?>