<?php
if (!defined( 'ABSPATH' )) { exit; }

function atec_wpmc_page_buffer_start()
{ 	 	
	// @codingStandardsIgnoreStart
	/* $_POST and $_SERVER is uncritical as it is only used for comparison */
	if (($_SERVER['REQUEST_METHOD']??'')!=='GET') { @header('X-Cache: SKIP:GET'); return; }
	// @codingStandardsIgnoreEnd
	
	//	if (str_contains($args,'/password-reset/') || str_contains($args,'/login/') || str_contains($args,'/wp-admin/')) { @header('X-Cache: SKIP/LOGIN'); return; }
	// this is already done in advanced cache
	
	global $wp_query;
	if ($wp_query->is_404 || $wp_query->is_search || $wp_query->is_login || $wp_query->is_admin) { @header('X-Cache: SKIP:IS_'); return; }
	
	if (class_exists('woocommerce' ) && (is_cart() || is_checkout() || is_account_page() || is_woocommerce())) { @header('X-Cache: SKIP:WOO'); return; }
	if (is_user_logged_in()) { @header('X-Cache: SKIP:LOGGED_IN'); return; }
	if (wp_doing_ajax()) { @header('X-Cache: SKIP:AJAX'); return; }

	ob_start(function($buffer) { return atec_wpmc_page_buffer_callback($buffer); });
 }
 
function atec_wpmc_minify($buffer)
{
	$search = ['/\>[^\S ]+/s',	'/[^\S ]+\</s','/(\s)+/s','/<!--(.|\s)*?-->/'];
	$replace = ['>','<','\\1',''];
	return preg_replace($search, $replace, $buffer);
}

function atec_wpmc_storeCache($args,$arr,$size,$suffix,$id)
{
	global $atec_wpmc_settings;
	$salt 		= $atec_wpmc_settings['salt']??'';
	$type 		= $atec_wpmc_settings['type']??'';
	
	$args 		= str_replace(['/','?'],['_','#'],$args);
	$prefix 	= 'atec_WPMC_'.$salt.'_';
	$key 		= $prefix.$args;
	if (in_array($type, ['APCu','Redis','Memcached'])) $hitKey=$prefix.'@'.$args;

	@require_once(__DIR__.'/atec-wpmc-connect.php'); atec_wpmc_connect($type,false);
	if ($type!=='APCu') 
	{
		global $atec_wpmc_source; 
		$s = $atec_wpmc_source[strtolower($type)]; 
		if (!in_array($type, ['APCu','Memcached'])) $serialized = serialize($arr);
	}

	switch ($type)
	{
		case 'APCu': $result=apcu_store($key,$arr); apcu_store($hitKey,0); break;
		case 'Redis': $result=$s->set($key,$serialized); $s->set($hitKey,0); break; 
		case 'Memcached': $result=$s->set($key,$arr); $s->set($hitKey,0); break;
		case 'SQLite':	
			$stmt=$s->prepare("INSERT INTO atec_wpmc (key, content) values (:key, :content);");
			$stmt->bindValue(':key', $args, SQLITE3_TEXT);
			$stmt->bindValue(':content', $serialized, SQLITE3_BLOB);
			$result=$stmt->execute();
			$stmt->close();
			break;
		case 'MongoDB':			
			try 
			{ 
				$bson	= new MongoDB\BSON\Binary($serialized, MongoDB\BSON\Binary::TYPE_GENERIC);
				$bulk 	= new MongoDB\Driver\BulkWrite;								
				$bulk->insert(['_id' => $args, 'content' => $bson]);
				$result = $s->executeBulkWrite('db.atec_wpmc', $bulk);
			}
			catch (MongoDB\Driver\Exception\Exception $e) { $result=false; };
			break;	
		case 'Database':
			global $wpdb;
			$insArr=array('key' =>$args, 'content'=>$serialized);
			// @codingStandardsIgnoreStart
			$result=$wpdb->insert($s, $insArr);
			// @codingStandardsIgnoreEnd			
			break;
		case 'File': 
			global $wp_filesystem;
			if (empty($wp_filesystem)) { @require_once (ABSPATH . '/wp-admin/includes/file.php'); WP_Filesystem(); }
			$result=$wp_filesystem->put_contents($s.$args.'.txt',$serialized); 
			break;
	}		
	
	if ($result)
	{
		$tableIdx 	= $atec_wpmc_settings['idx']??'';
		if (!isset($wpdb)) global $wpdb;
		// @codingStandardsIgnoreStart
		$insArr=array('key'=>$args, 'size' =>$size, 'suffix' =>$suffix, 'pid'=>$id);
		$wpdb->insert($tableIdx, $insArr);
		// @codingStandardsIgnoreEnd			
	}

}

function atec_wpmc_page_buffer_callback($buffer)
{
	if (strlen($buffer)<1024) return $buffer;
	if (is_user_logged_in()) { @header('X-Cache: SKIP/LOGGED_IN'); return $buffer; }
	
	global $atec_wpmc_settings;
	if (($isCat=is_category()) || ($isTag=is_tag()))
	{
		$id = get_queried_object()->term_id;
		if ($isCat) { $tmp=get_category($id); $suffix='c'; }
		else { $tmp=get_tag($id); $suffix='t'; }
		if (is_wp_error($tmp) || is_null($tmp)) { @header('X-Cache: FAIL-CAT/TAG_EMPTY'); return $buffer; }
	}
	else
	{
		$id 		= get_the_ID();
		$type 	= get_post_type($id);
		$suffix = $type==='page'?'a':($type==='post'?'o':false);
		if (!$suffix) 
		{ 
			if (in_array($type,['attachment','revision','nav_menu_item','wp_template','wp_template_part'])) { @header('X-Cache: SKIP:TYPE_'.strtoupper($type)); return $buffer; }
			if (!($atec_wpmc_settings['custom']??false)) { @header('X-Cache: SKIP:TYPE_'.$type); return $buffer; }
			$suffix = 'x';
		}
	}
	
	@header('X-Cache: STORE -> '.esc_attr($suffix.'|'.$id));
	
	$gzip = false; $compressed=''; $debug=''; $debugLen=0;
	if (($atec_wpmc_settings['debug']??false))
	{
		$debug=atec_wpmc_minify('	
			<script id="atec_wpmc_debug_script">
			console.log(\'APCu Cache: HIT '.esc_attr(get_locale()).' | '.esc_attr($suffix.'|'.$id).'\');
			var elemDiv = document.createElement("div");
			elemDiv.innerHTML="🟢";
			elemDiv.id="atec_wpmc_debug";
			elemDiv.style.cssText = "position:absolute;top:3px;width:8px;height:8px;font-size:8px;left:3px;z-index:99999;";
			document.body.appendChild(elemDiv);
			setTimeout(()=>{ const elem=document.getElementById("atec_wpmc_debug"); if (elem) elem.remove(); }, 3000);
			const elem=document.getElementById("atec_wpmc_debug_script"); if (elem) elem.remove();
		</script>');
		$debugLen=strlen($debug);
	}
	if ($atec_wpmc_settings['minify']??false) { $buffer=atec_wpmc_minify($buffer); }
	$powered='<p style="font-size:0; margin:0;">Powered by <a href="https://wpmegacache.com/">wpmegacache.com</a></p>';
	if (function_exists('gzencode')) 
	{ 
		//$charset=$atec_wpmc_settings['charset']??'UTF-8';
		$compressed = gzencode($buffer.$debug.$powered); $gzip=true; 
	}
	$size=$gzip?strlen($compressed):strlen($buffer)+$debugLen;
	$args = add_query_arg(null,null);
	atec_wpmc_storeCache($args,array($gzip,$gzip?$compressed:$buffer.$debug.$powered),$size,$suffix,$id);
	unset($compressed);
	return $buffer;
}

add_action('template_redirect', 'atec_wpmc_page_buffer_start');
?>