<?php
if (!defined('ABSPATH')) { exit; }
/**
* Plugin Name:  Mega Cache
* Plugin URI: https://wpmegacache.com/
* Description: Page cache to improve site speed. Reduces CPU time, memory usage and number of database queries by an order of magnitude.
* Version: 1.0.22
* Requires at least: 5.2
* Tested up to: 6.6.2
* Requires PHP: 7.4
* Author: Chris Ahrweiler
* Author URI: https://wpmegacache.com
* License: GPL2
* License URI:  https://www.gnu.org/licenses/gpl-2.0.html
* Text Domain:  mega-cache
*/

wp_cache_set('atec_wpmc_version','1.0.22');

$atec_wpmc_settings=get_option('atec_WPMC_settings',[]);
$atec_wpmc_extensions=get_option('atec_WPMC_extensions',[]);

function atec_wpmc_settings($opt): bool { global $atec_wpmc_settings; return $atec_wpmc_settings[$opt]??null==true; }
function atec_wpmc_key($salt,$suffix,$id): string { return 'atec_WPMC_'.$salt.'_'.$suffix.'_'.$id; }
function atec_wpmc_hit_key($salt,$suffix,$id): string { return 'atec_WPMC_'.$salt.'_h_'.$suffix.'_'.$id; }

if (is_admin()) 
{
	register_activation_hook( __FILE__, function() { @require_once(__DIR__.'/includes/atec-wpmc-activation.php'); });
	register_deactivation_hook( __FILE__, function() { @require_once(__DIR__.'/includes/atec-wpmc-deactivation.php'); });
	
	if (atec_wpmc_settings('cache'))
	{ 			
		function atec_wpmc_admin_footer_function($content): string
		{
			$yes='dashicons dashicons-yes-alt'; 
			$style='padding-top: 5px; font-size: 16px; color:green;';
			$icon=plugin_dir_url( __FILE__ ) . 'assets/img/atec-group/atec_wpmc_icon.svg';
			// @codingStandardsIgnoreStart
			// Image is not an attachement
			$content.=' | <sub><img alt="Mega Cache" src="'.esc_url($icon).'" style="height: 20px; vertical-align: bottom;"> MCache <span style="'.esc_html($style).'" class="'.esc_html($yes).'"></span>';
			// @codingStandardsIgnoreEnd
			$content.='</sub>';
			return $content; 
		}
		add_action('admin_footer_text', 'atec_wpmc_admin_footer_function');
	}

	if (!defined('ATEC_ADMIN_INC')) @require_once(__DIR__.'/includes/atec-admin.php');
	add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'atec_plugin_settings' );
	
	if (!defined('ATEC_INIT_INC')) @require_once('includes/atec-init.php');
	add_action('admin_menu', function() { atec_wp_menu(__FILE__,'atec_wpmc','Mega Cache',true); });

	global $atec_active_slug;
   	if (in_array($atec_active_slug=atec_get_slug(), ['atec_group','atec_wpmc'])) require_once(__DIR__.'/includes/atec-wpmc-install.php');
	
	add_action('init', function() 
	{ 
		if (preg_match('/page=atec_wpmc|options\.php/', atec_query())) @require_once(__DIR__.'/includes/atec-wpmc-register-settings.php');
			
		if (atec_wpmc_settings('cache')) 
		{ 
			function atec_wpmc_admin_bar($wp_admin_bar): void
			{
				$link = get_admin_url().'admin.php?page=atec_wpmc&action=delete_all&nav=MCache&_wpnonce='.esc_attr(wp_create_nonce('atec_wpmc_nonce'));
				$style = 'vertical-align: bottom; margin:7px 4px 7px 0;';
				// @codingStandardsIgnoreStart
				// Image is not an attachement
				$args = array('id' => 'atec_wpmc_admin_bar', 'title' => '
					<span title="'.__('Flush MCache','mega-cache').'" style="font-size:12px;">
						<img src="'. plugins_url( '/assets/img/atec_wpmc_icon_admin.svg', __FILE__ ) .'" style="height:18px; '.esc_attr($style).'">Flush
					</span>', 'href' => $link );
				// @codingStandardsIgnoreEnd
				$wp_admin_bar->add_node($args);
			}
			add_action('admin_bar_menu', 'atec_wpmc_admin_bar', PHP_INT_MAX);
			
			require_once(__DIR__.'/includes/atec-wpmc-pcache-tools.php');
			add_action( 'after_switch_theme', 'atec_wpmc_delete_page_cache_all');
			add_action( 'activated_plugin', 'atec_wpmc_delete_page_cache_all');
			add_action( 'deactivated_plugin', 'atec_wpmc_delete_page_cache_all');
			add_action( 'wp_ajax_edit_theme_plugin_file', 'atec_wpmc_delete_page_cache_all');				
			add_action( 'wp_update_nav_menu', 'atec_wpmc_delete_page_cache_all');		
			add_action( 'permalink_structure_changed', 'atec_wpmc_delete_page_cache_all');		
		}
	});

}

add_action('init', function() 
{
	if (atec_wpmc_settings('cache'))
	{
		if (current_user_can('edit_posts')) 
		{
			@require_once(__DIR__.'/includes/atec-wpmc-pcache-tools.php');
			add_action( 'delete_category', 'atec_wpmc_update_category_cache');
			add_action( 'delete_term', 'atec_wpmc_update_tag_cache');
			add_action( 'post_updated', 'atec_wpmc_update_page_cache',10,3);
		}
		@require_once(__DIR__.'/includes/atec-wpmc-pcache-cleanup.php');
	}
});

?>